@@ -21,10 +21,10 @@ from sklearn import pipeline
 from sklearn.compose import ColumnTransformer
 from sklearn.preprocessing import OneHotEncoder
 
-from ._container import CommonONNXModelContainer
-from ._utils import sklearn_installed
+from ._container import CommonONNXModelContainer, CommonSparkMLModelContainer
+from ._utils import sklearn_installed, sparkml_installed
 from .operator_converters import constants
-from .supported import get_sklearn_api_operator_name, get_onnxml_api_operator_name
+from .supported import get_sklearn_api_operator_name, get_onnxml_api_operator_name, get_sparkml_api_operator_name
 
 do_not_merge_columns = tuple(filter(lambda op: op is not None, [OneHotEncoder, ColumnTransformer]))
 
@@ -53,59 +53,50 @@ def parse_sklearn_api_model(model, extra_config={}):
     scope = topology.declare_scope("__root__")
 
     # Declare input variables.
-    inputs = []
-    n_inputs = extra_config[constants.N_INPUTS] if constants.N_INPUTS in extra_config else 1
-    if constants.INPUT_NAMES in extra_config:
-        assert n_inputs == len(extra_config[constants.INPUT_NAMES])
-    if constants.TEST_INPUT in extra_config:
-        from onnxconverter_common.data_types import FloatTensorType, DoubleTensorType, Int32TensorType, Int64TensorType
-
-        test_input = extra_config[constants.TEST_INPUT] if n_inputs > 1 else [extra_config[constants.TEST_INPUT]]
-        for i in range(n_inputs):
-            input = test_input[i]
-            input_name = (
-                extra_config[constants.INPUT_NAMES][i] if constants.INPUT_NAMES in extra_config else "input_{}".format(i)
-            )
-            if input.dtype == np.float32:
-                input_type = FloatTensorType(input.shape)
-            elif input.dtype == np.float64:
-                input_type = DoubleTensorType(input.shape)
-            elif input.dtype == np.int32:
-                input_type = Int32TensorType(input.shape)
-            elif input.dtype == np.int64:
-                input_type = Int64TensorType(input.shape)
-            else:
-                raise RuntimeError(
-                    "Type {} not supported. Please fill an issue on https://github.com/microsoft/hummingbird/.".format(
-                        type(input.dtype)
-                    )
-                )
-            inputs.append(scope.declare_local_variable(input_name, type=input_type))
-    else:
-        # We have no information on the input. Sklearn always gets as input a single dataframe,
-        # therefore by default we start with a single `input` variable
-        input_name = extra_config[constants.INPUT_NAMES][0] if constants.TEST_INPUT in extra_config else "input"
-        inputs.append(scope.declare_local_variable(input_name))
-
-    # The object raw_model_container is a part of the topology we're going to return.
-    # We use it to store the inputs of the scikit-learn's computational graph.
-    for variable in inputs:
-        raw_model_container.add_input(variable)
+    inputs = _declare_input_variables(scope, raw_model_container, extra_config)
 
     # Parse the input scikit-learn model into its scope with the topology.
     # Get the outputs of the model.
     outputs = _parse_sklearn_api(scope, model, inputs)
 
-    # Use the output names specified by the user, if any
-    if constants.OUTPUT_NAMES in extra_config:
-        assert len(extra_config[constants.OUTPUT_NAMES]) == len(outputs)
-        for i in range(len(outputs)):
-            outputs[i].raw_name = extra_config[constants.OUTPUT_NAMES][i]
+    # Declare output variables.
+    _declare_output_variables(raw_model_container, extra_config, outputs)
 
-    # The object raw_model_container is a part of the topology we're going to return.
-    # We use it to store the outputs of the scikit-learn's computational graph.
-    for variable in outputs:
-        raw_model_container.add_output(variable)
+    return topology
+
+
+def parse_sparkml_api_model(model, extra_config={}):
+    """
+    Puts *Spark-ML* object into an abstract representation so that our framework can work seamlessly on models created
+    with different machine learning tools.
+
+    Args:
+        model: A model object in Spark-ML format
+
+    Returns:
+        A `onnxconverter_common.topology.Topology` object representing the input model
+    """
+    assert model is not None, "Cannot convert a mode of type None."
+
+    raw_model_container = CommonSparkMLModelContainer(model)
+
+    # Declare a computational graph. It will become a representation of
+    # the input Spark-ML model after parsing.
+    topology = Topology(raw_model_container)
+
+    # Declare an object to provide variables' and operators' naming mechanism.
+    # One global scope is enough for parsing Spark-ML models.
+    scope = topology.declare_scope("__root__")
+
+    # Declare input variables.
+    inputs = _declare_input_variables(scope, raw_model_container, extra_config)
+
+    # Parse the input Spark-ML model into its scope with the topology.
+    # Get the outputs of the model.
+    current_op_outputs, _ = _parse_sparkml_api(scope, model, inputs)
+
+    # Declare output variables.
+    _declare_output_variables(raw_model_container, extra_config, current_op_outputs)
 
     return topology
 
@@ -157,6 +148,64 @@ def parse_onnx_api_model(model):
     return topology
 
 
+def _declare_input_variables(scope, raw_model_container, extra_config):
+    # Declare input variables.
+    inputs = []
+    n_inputs = extra_config[constants.N_INPUTS] if constants.N_INPUTS in extra_config else 1
+    if constants.INPUT_NAMES in extra_config:
+        assert n_inputs == len(extra_config[constants.INPUT_NAMES])
+    if constants.TEST_INPUT in extra_config:
+        from onnxconverter_common.data_types import FloatTensorType, DoubleTensorType, Int32TensorType, Int64TensorType
+
+        test_input = extra_config[constants.TEST_INPUT] if n_inputs > 1 else [extra_config[constants.TEST_INPUT]]
+        for i in range(n_inputs):
+            input = test_input[i]
+            input_name = (
+                extra_config[constants.INPUT_NAMES][i] if constants.INPUT_NAMES in extra_config else "input_{}".format(i)
+            )
+            if input.dtype == np.float32:
+                input_type = FloatTensorType(input.shape)
+            elif input.dtype == np.float64:
+                input_type = DoubleTensorType(input.shape)
+            elif input.dtype == np.int32:
+                input_type = Int32TensorType(input.shape)
+            elif input.dtype == np.int64:
+                input_type = Int64TensorType(input.shape)
+            else:
+                raise NotImplementedError(
+                    "Type {} not supported. Please fill an issue on https://github.com/microsoft/hummingbird/.".format(
+                        input.dtype
+                    )
+                )
+            inputs.append(scope.declare_local_variable(input_name, type=input_type))
+    else:
+        # We have no information on the input. Sklearn/Spark-ML always gets as input a single dataframe,
+        # therefore by default we start with a single `input` variable
+        input_name = extra_config[constants.INPUT_NAMES][0] if constants.TEST_INPUT in extra_config else "input"
+        inputs.append(scope.declare_local_variable(input_name))
+
+    # The object raw_model_container is a part of the topology we're going to return.
+    # We use it to store the inputs of the Sklearn/Spark-ML's computational graph.
+    for variable in inputs:
+        raw_model_container.add_input(variable)
+
+    return inputs
+
+
+def _declare_output_variables(raw_model_container, extra_config, outputs):
+    # Declare output variables.
+    # Use the output names specified by the user, if any
+    if constants.OUTPUT_NAMES in extra_config:
+        assert len(extra_config[constants.OUTPUT_NAMES]) == len(outputs)
+        for i in range(len(outputs)):
+            outputs[i].raw_name = extra_config[constants.OUTPUT_NAMES][i]
+
+    # The object raw_model_container is a part of the topology we're going to return.
+    # We use it to store the outputs of the Sklearn/Spark-ML's computational graph.
+    for variable in outputs:
+        raw_model_container.add_output(variable)
+
+
 def _parse_sklearn_api(scope, model, inputs):
     """
     This is a delegate function adding the model to the input scope.
@@ -222,6 +271,24 @@ def _parse_sklearn_pipeline(scope, model, inputs):
     return inputs
 
 
+def _parse_sparkml_pipeline(scope, model, all_outputs):
+    """
+    The basic ideas of Spark-ML parsing:
+        1. Sequentially go though all stages defined in the considered
+           Spark-ML pipeline passing all outputs that has been generated so far
+           as the input. Operator will pick which inputs to operates on.
+        2. The output variables of one stage will be fed into its next
+           stage as the inputs.
+    :param scope: Scope object defined in _topology.py
+    :param model: Spark-ML pipeline object
+    :param all_outputs: A list of Variable objects
+    :return: A list of output variables produced by the input pipeline
+    """
+    for step in model.stages:
+        current_op_outputs, all_outputs = _parse_sparkml_api(scope, step, all_outputs)
+    return current_op_outputs, all_outputs
+
+
 def _parse_sklearn_feature_union(scope, model, inputs):
     """
     Taken from https://github.com/onnx/sklearn-onnx/blob/9939c089a467676f4ffe9f3cb91098c4841f89d8/skl2onnx/_parse.py#L199.
@@ -352,6 +419,18 @@ def _build_sklearn_api_parsers_map():
     return map_parser
 
 
+def _build_sparkml_api_parsers_map():
+    # Parsers for edge cases are going here.
+    from pyspark.ml.pipeline import PipelineModel
+
+    map_parser = {
+        PipelineModel: _parse_sparkml_pipeline,
+        # More parsers will go here
+    }
+
+    return map_parser
+
+
 def _parse_onnx_api(scope, model, inputs):
     """
     This function handles all input ONNX models.
@@ -407,6 +486,69 @@ def _parse_onnx_single_operator(scope, operator):
         this_operator.outputs.append(variable)
 
 
+def _parse_sparkml_api(scope, model, inputs):
+    """
+    This function handles all input Spark-ML models.
+
+    Args:
+        scope: The ``onnxconverter_common.topology.Scope`` where the model will be added
+        model: A Spark-ML model object
+        inputs: A list of `onnxconverter_common.topology.Variable`s
+
+    Returns:
+        A list of output `onnxconverter_common.topology.Variable` which will be passed to next stage
+    """
+    tmodel = type(model)
+    if tmodel in sparkml_api_parsers_map:
+        outputs = sparkml_api_parsers_map[tmodel](scope, model, inputs)
+    else:
+        outputs = _parse_sparkml_single_operator(scope, model, inputs)
+    return outputs
+
+
+def _parse_sparkml_single_operator(scope, operator, all_inputs):
+    """
+    This function handles the parsing of all Spark-ML operators.
+
+    Args:
+        scope: The ``onnxconverter_common.topology.Scope`` where the model will be added
+        model: A Spark-ML operator
+        all_inputs: A list of `onnxconverter_common.topology.Variable`s
+    """
+    import inspect
+
+    if isinstance(operator, str):
+        raise RuntimeError("Parameter operator must be an object not a " "string '{0}'.".format(operator))
+
+    alias = get_sparkml_api_operator_name(type(operator))
+    this_operator = scope.declare_local_operator(alias, operator)
+
+    if hasattr(operator, "getInputCol") and callable(operator.getInputCol):
+        this_operator.inputs = [i for i in all_inputs if i.raw_name == operator.getInputCol()]
+    elif hasattr(operator, "getInputCols") and callable(operator.getInputCols):
+        temp = {i.raw_name: i for i in all_inputs if i.raw_name in operator.getInputCols()}
+        this_operator.inputs = [temp[i] for i in operator.getInputCols()]
+    elif operator.hasParam("featuresCol"):
+        col_name = [param[1] for param in operator.extractParamMap().items() if param[0].name == "featuresCol"][0]
+        this_operator.inputs = [i for i in all_inputs if i.raw_name == col_name]
+    else:
+        print(operator.getParam("featuresCol"))
+        raise RuntimeError("Unable to determine inputs for the Spark-ML operator: {}".format(type(operator)))
+
+    if hasattr(operator, "getOutputCol") and callable(operator.getOutputCol):
+        variable = scope.declare_local_variable(operator.getOutputCol())
+        this_operator.outputs.append(variable)
+    elif hasattr(operator, "getOutputCols") and callable(operator.getOutputCols):
+        for output_col in operator.getOutputCols():
+            variable = scope.declare_local_variable(output_col)
+            this_operator.outputs.append(variable)
+    else:
+        variable = scope.declare_local_variable("variable")
+        this_operator.outputs.append(variable)
+
+    return this_operator.outputs, all_inputs + this_operator.outputs
+
+
 def _remove_zipmap(node_list):
     """
     Method used to remove ZipMap operators in the graph.
@@ -455,7 +597,7 @@ def _fetch_input_slice(scope, inputs, column_indices):
         len(inputs) == 1
         and len(column_indices) == 1
         and inputs[0].type is not None
-        and (inputs[0].type.shape == 1 or inputs[0].type.shape[1] == 1)
+        and (len(inputs[0].type.shape) == 1 or inputs[0].type.shape[1] == 1)
     ):
         return inputs
     array_feature_extractor_operator = scope.declare_local_operator("SklearnArrayFeatureExtractor")
@@ -515,6 +657,9 @@ def _get_column_index(i, inputs):
                 )
             end += rel_end
     else:
+        assert isinstance(
+            i, str
+        ), "Type {} not supported. Please fill an issue on https://github.com/microsoft/hummingbird/.".format(type(i))
         for ind, inp in enumerate(inputs):
             if inp.onnx_name == i:
                 return ind, 0
@@ -561,3 +706,5 @@ def _get_column_indices(indices, inputs, multiple=False):
 # Registered API parsers.
 if sklearn_installed():
     sklearn_api_parsers_map = _build_sklearn_api_parsers_map()
+if sparkml_installed():
+    sparkml_api_parsers_map = _build_sparkml_api_parsers_map()
