@@ -31,6 +31,7 @@ from .sklearn import discretizer as sklearn_discretizer  # noqa: E402
 from .sklearn import gbdt  # noqa: E402
 from .sklearn import iforest  # noqa: E402
 from .sklearn import imputer  # noqa: E402
+from .sklearn import kneighbors  # noqa: E402
 from .sklearn import linear as sklearn_linear  # noqa: E402
 from .sklearn import mlp as sklearn_mlp  # noqa: E402
 from .sklearn import nb as sklearn_nb  # noqa: E402
@@ -42,18 +43,6 @@ from .sklearn import scaler as sklearn_scaler  # noqa: E402
 from .sklearn import sv  # noqa: E402
 from . import lightgbm  # noqa: E402
 from . import xgb  # noqa: E402
-
-
-__pdoc__ = {}
-__pdoc__["hummingbird.operator_converters._array_feature_extractor_implementations"] = True
-__pdoc__["hummingbird.operator_converters._decomposition_implementations"] = True
-__pdoc__["hummingbird.operator_converters._discretizer_implementations"] = True
-__pdoc__["hummingbird.operator_converters._gbdt_commons"] = True
-__pdoc__["hummingbird.operator_converters._linear_implementations"] = True
-__pdoc__["hummingbird.operator_converters._mlp_implementations"] = True
-__pdoc__["hummingbird.operator_converters._nb_implementations"] = True
-__pdoc__["hummingbird.operator_converters._normalizer_implementations"] = True
-__pdoc__["hummingbird.operator_converters._one_hot_encoder_implementations"] = True
-__pdoc__["hummingbird.operator_converters._scaler_implementations"] = True
-__pdoc__["hummingbird.operator_converters._tree_commons"] = True
-__pdoc__["hummingbird.operator_converters._tree_implementations"] = True
+from .sparkml import discretizer  # noqa: E402
+from .sparkml import linear  # noqa: E402
+from .sparkml import vector_assembler  # noqa: E402
