@@ -14,6 +14,7 @@ import torch
 
 from .. import constants
 from .._base_operator import BaseOperator
+from .._pipeline_implementations import Concat
 
 
 class Cast(BaseOperator, torch.nn.Module):
@@ -22,24 +23,21 @@ class Cast(BaseOperator, torch.nn.Module):
 
         assert to_type is not None
 
-        self.to_type = to_type
+        self._to_type = to_type
 
     def forward(self, x):
-        if self.to_type == 1:  # Cast to float
+        if self._to_type == 1:  # Cast to float
             return x.float()
-        if self.to_type == 7:  # Cast to long
+        elif self._to_type == 7:  # Cast to long
             return x.long()
-
-
-class Concat(BaseOperator, torch.nn.Module):
-    def __init__(self):
-        super(Concat, self).__init__()
-
-    def forward(self, *x):
-        if len(x[0].shape) > 1:
-            return torch.cat(x, dim=1)
+        elif self._to_type == 11:  # Cast to double
+            return x.double()
         else:
-            return torch.stack(x, dim=1)
+            raise RuntimeError(
+                "Cast to ONNX type {} not supported yet. Please fill an issue at https://github.com/microsoft/hummingbird".format(
+                    self._to_type
+                )
+            )
 
 
 class Reshape(BaseOperator, torch.nn.Module):
