@@ -28,6 +28,8 @@ HistGradientBoostingRegressor,
 IsolationForest,
 KernelPCA,
 KBinsDiscretizer,
+KNeighborsClassifier,
+KNeighborsRegressor,
 LinearRegression,
 LinearSVC,
 LogisticRegression,
@@ -67,7 +69,14 @@ XGBRegressor
 from collections import defaultdict
 
 from .exceptions import MissingConverter
-from ._utils import torch_installed, sklearn_installed, lightgbm_installed, xgboost_installed, onnx_runtime_installed
+from ._utils import (
+    torch_installed,
+    sklearn_installed,
+    lightgbm_installed,
+    xgboost_installed,
+    onnx_runtime_installed,
+    sparkml_installed,
+)
 
 
 def _build_sklearn_operator_list():
@@ -116,6 +125,10 @@ def _build_sklearn_operator_list():
         # Matrix decomposition transformers
         from sklearn.decomposition import PCA, KernelPCA, FastICA, TruncatedSVD
 
+        # KNeighbors models
+        from sklearn.neighbors import KNeighborsClassifier
+        from sklearn.neighbors import KNeighborsRegressor
+
         # Preprocessing
         from sklearn.preprocessing import (
             Binarizer,
@@ -161,6 +174,8 @@ def _build_sklearn_operator_list():
             # Other models
             BernoulliNB,
             GaussianNB,
+            KNeighborsClassifier,
+            KNeighborsRegressor,
             MLPClassifier,
             MLPRegressor,
             MultinomialNB,
@@ -197,6 +212,27 @@ def _build_sklearn_operator_list():
     return []
 
 
+def _build_sparkml_operator_list():
+    """
+    List all suported SparkML operators.
+    """
+    if sparkml_installed():
+        from pyspark.ml.classification import LogisticRegressionModel
+        from pyspark.ml.feature import Bucketizer, VectorAssembler
+
+        supported_ops = [
+            # Featurizers
+            Bucketizer,
+            VectorAssembler,
+            # Linear Models
+            LogisticRegressionModel,
+        ]
+
+        return supported_ops
+
+    return []
+
+
 def _build_xgboost_operator_list():
     """
     List all suported XGBoost (Sklearn API) operators.
@@ -296,6 +332,14 @@ def _build_onnxml_api_operator_name_map():
     return {k: "ONNXML" + k for k in onnxml_operator_list if k is not None}
 
 
+def _build_sparkml_api_operator_name_map():
+    """
+    Associate Spark-ML with the operator class names.
+    If two Spark-ML models share a single name, it means they are equivalent in terms of conversion.
+    """
+    return {k: "SparkML" + k.__name__ if hasattr(k, "__name__") else k for k in sparkml_operator_list if k is not None}
+
+
 def get_sklearn_api_operator_name(model_type):
     """
     Get the operator name for the input model type in *scikit-learn API* format.
@@ -328,40 +372,59 @@ def get_onnxml_api_operator_name(model_type):
     return onnxml_api_operator_name_map[model_type]
 
 
+def get_sparkml_api_operator_name(model_type):
+    """
+    Get the operator name for the input model type in *Spark-ML API* format.
+
+    Args:
+        model_type: A Spark-ML model object (e.g., LogisticRegression)
+
+    Returns:
+        A string which stands for the type of the input model in the Hummingbird conversion framework.
+        None if the model_type is not supported
+    """
+    if model_type not in sparkml_api_operator_name_map:
+        return None
+    return sparkml_api_operator_name_map[model_type]
+
+
 # Supported operators.
 sklearn_operator_list = _build_sklearn_operator_list()
 xgb_operator_list = _build_xgboost_operator_list()
 lgbm_operator_list = _build_lightgbm_operator_list()
 onnxml_operator_list = _build_onnxml_operator_list()
+sparkml_operator_list = _build_sparkml_operator_list()
 
 sklearn_api_operator_name_map = _build_sklearn_api_operator_name_map()
 onnxml_api_operator_name_map = _build_onnxml_api_operator_name_map()
-
+sparkml_api_operator_name_map = _build_sparkml_api_operator_name_map()
 
 # Supported backends.
 backends = _build_backend_map()
 
 # Supported configurations settings accepted by Hummingbird are defined below.
-N_FEATURES = "n_features"
-"""Number of features expected in the input data."""
-
+# Please check `test.test_extra_conf.py` for examples on how to use these.
 TREE_IMPLEMENTATION = "tree_implementation"
 """Which tree implementation to use. Values can be: gemm, tree-trav, perf_tree_trav."""
 
 ONNX_OUTPUT_MODEL_NAME = "onnx_model_name"
 """For ONNX models we can set the name of the output model."""
 
-ONNX_INITIAL_TYPES = "onnx_initial_types"
-"""For ONNX models we can explicitly set the input types and shapes."""
-
 ONNX_TARGET_OPSET = "onnx_target_opset"
-"""For ONNX models we can set the target opset to use. 9 by default."""
+"""For ONNX models we can set the target opset to use. 11 by default."""
 
 INPUT_NAMES = "input_names"
-"""Set the names of the inputs. Assume that the numbers onf inputs_names is equal to the number of inputs."""
+"""Set the names of the inputs. Assume that the numbers of inputs_names is equal to the number of inputs."""
 
 OUTPUT_NAMES = "output_names"
 """Set the names of the outputs."""
 
 CONTAINER = "container"
-"""Whether to return the container for Sklearn API or just the model"""
+"""Boolean used to chose whether to return the container for Sklearn API or just the model."""
+
+N_THREADS = "n_threads"
+"""Select how many threads to use for scoring. This paremeter will set the number of intra-op threads.
+Inter-op threads are by default set to 1 in Hummingbird. Check `tests.test_extra_conf.py` for usage examples."""
+
+BATCH_SIZE = "batch_size"
+"""Select whether to partition the input dataset at inference time in N batch_size partitions."""
